# outer __init__.py

__version__ = '0.7.2'

__api_mpy_path__ = 'data/api/mpython.json'
__api_node_path__ = 'data/api/node.json'
__serial_config_path__ = 'data/serial/config.json'

import pyesp.pyesp
import pyesp.run



#
import os
import sys
import argparse
import json
from pyesp import __version__
from pyesp import __serial_config_path__
from pyesp.pyesp import ESP


serial_config_path = os.path.join(os.path.dirname(__file__), __serial_config_path__)


def loadconfig():
    with open(serial_config_path, 'r') as f:
        data = json.loads(f.read())
        if not len(data.keys()):
            raise BaseException('serial port config file empty')
    return data


def saveconfig(data):
    jobj = json.dumps( data )
    with open(serial_config_path, 'w') as f:
        f.write(jobj)


def createArgParser():
    """ Create arguments cmd line parser """
    _prog = "pyesp"
    desctext = '"%s v%s"' % ( _prog, __version__ )
    epitext = 'Find more info visit http://hobby-research.at.ua ; https://github.com/LeftRadio/pyesp'

    try:
        opt = loadconfig()
        serialconfig = '%s,%s,%s,%s,%s,%s,%s' % ( \
            opt['portName'], opt['baudRate'], opt['dataBits'], \
            opt['parity'], opt['stopBits'],
            opt['flowControl'], opt['linedelay'] )
    except Exception as e:
        raise BaseException('error loading default serial port config file')

    parser = argparse.ArgumentParser( description = desctext,
                                      epilog = epitext,
                                      prog = _prog )

    parser.add_argument ( '-V', '--version', action='version',
                            help = '\tversion for tool',
                            version = __version__ )
    parser.add_argument ( '-a', '--api', dest = 'api_userfile', type = str,
                            default = '',
                            metavar = 'FILE',
                            help = '\t external api file' )
    parser.add_argument ( '-c', '--command', dest = 'command', type = str,
                            default = '',
                            metavar = 'CMD',
                            help = '' )
    parser.add_argument ( '-d', '--data', dest = 'data', type = str,
                            default = '',
                            metavar = 'DATA',
                            help = 'command data, ignored for "file*" commands' )
    parser.add_argument ( '-f', '--files', dest = 'files', type = str,
                            default = '',
                            metavar = 'LIST',
                            help = '' )
    parser.add_argument ( '-p', '--platform', dest = 'platform', type = str,
                            default = 'MPY',
                            metavar = 'PLT',
                            help = ( '\t platform for ESP - '
                                     '{ MPY, NODE } '
                                     '[default MPY]' ) )
    parser.add_argument ( '-s', '--serial', dest = 'serialconfig', type = str,
                            default = serialconfig,
                            metavar = 'PARAMS',
                            help = ( '\t serial port parametrs - '
                                     'NAME,BOUD,DATABITS,PARITY,STOPBITS,FLOW,LINEDELAY '
                                     '[default %s], '
                                     'linedelay is specified in milliseconds' % serialconfig ) )
    parser.add_argument( '-v', '--verbose', action='store_true', default = False,
                            help = ( '\t increase output verbosity '
                                     '[default False]') )
    return parser


readend = False

#
def read_callback(data):
    global readend
    if not readend:
        readend = ('END' in data)
    print(data)


def main():
    #
    global readend
    #
    parser = createArgParser()
    args = parser.parse_args()
    #
    if not len(sys.argv[1:]):
        parser.print_help()
        print('\r\nNo arguments, nothing remains :(, we leave...')
        return
    #
    try:
        sc = args.serialconfig.split(',')
        serialconfig = {
            'portName': str(sc[0]),
            'baudRate': int(sc[1]),
            'dataBits': int(sc[2]),
            'parity': int(sc[3]),
            'stopBits': int(sc[4]),
            'flowControl': int(sc[5]),
            'linedelay': int(sc[6]) }
    except Exception as e:
        raise BaseException('error in serial config parametrs')
    #
    esp = ESP( serialconfig, args.platform, args.api_userfile )
    #
    print('-'*80)
    print('Platforms API location: ')
    print('MicroPython: ', ESP._api_file['MPY'])
    print('NodeMCU: ', ESP._api_file['NODE'])
    print('-'*80)
    #
    if args.files:
        if args.command == 'filewrite':
            pass
        elif args.command == 'fileread':
            pass

        for f in args.files.split(','):
            print('-'*120)
            print('read: ', f)
            print('-'*120)
            esp.send( args.command, name=f, data=args.data, callback=read_callback )
            while not readend:
                esp.processEvents()
            readend = False
    else:
        esp.send( args.command, data=args.data, callback=read_callback )
        while not readend:
            esp.processEvents()

    #
    saveconfig( esp.serialconfig )
    #
    print('--- Press ENTER to exit ---')
    input()
    sys.exit(True)


if __name__ == '__main__':
    main()



#
import os
import threading
from queue import Queue
from time import time, sleep
from PyQt5.QtCore import Qt, QCoreApplication, QIODevice, QObject, pyqtSignal, pyqtSlot
from PyQt5.QtSerialPort import QSerialPort, QSerialPortInfo
from pyesp import __api_mpy_path__, __api_node_path__


class SerialPort(QSerialPort):
    """docstring for SerialPort"""

    readline_signal = pyqtSignal(str)
    writeline_signal = pyqtSignal(str)

    def __init__(self, serial_config, logger=None, parent=None):
        super(SerialPort, self).__init__(parent)
        self.writeline_data = ''
        self.readline_ready = False
        self.readline_data = ''
        self.workerbreak = False
        self.apply_settings(serial_config)
        #
        self.readyRead.connect(self.ready_read)
        self.writeline_signal.connect(self.write_data)
        self.log = logger
        # --- queue, threads, worker
        self.nqueue = Queue()
        t = threading.Thread(target=self.worker)
        t.daemon = True  # thread dies when main thread exits.
        t.start()

    def apply_settings(self, settings):
        self.setPortName(settings['portName'])
        self.setBaudRate(settings['baudRate'])
        self.setDataBits(settings['dataBits'])
        self.setParity(settings['parity'])
        self.setStopBits(settings['stopBits'])
        self.setFlowControl(settings['flowControl'])
        self.linedelay = settings['linedelay']

    def open(self):
        if not self.isOpen():
            return super().open(QIODevice.ReadWrite)
        return True

    def close(self):
        if self.isOpen():
            return super().close()
        return True

    def transaction(self, data, callback=None):
        if not self.open():
            raise OSError('cant open port - '+self.portName())
        #
        for s in data.split('\r\n'):
            s = s + '\r\n'
            # write line
            self.write(bytearray(s, 'utf-8'))
            #
            if not self.waitForBytesWritten(100):
                raise BaseException('error write data - ' + str(data))
            #
            # read response
            if not self.waitForReadyRead(100):
                raise BaseException('transaction response timeout error')
            #
            response = self.readAll()
            #
            while self.waitForReadyRead(100):
                response += self.readAll()

            try:
                response = str(response, 'utf-8')
            except UnicodeDecodeError:
                response = 'UnicodeDecodeError'
            #
            start_index = response.find(s)
            if start_index != -1:
                response = response[start_index+len(s):]
            #
            if callback:
                callback(response)
            #
            sleep(self.linedelay/1000)
        #
        self.close()

    @pyqtSlot(str)
    def write_data(self, data):
        if self.open():
            if self.write(bytearray(data, 'windows-1251')) != 0:
                self.workerbreak = False
                return
        self.workerbreak = True

    @pyqtSlot()
    def ready_read(self):
        try:
            data = self.readAll()
            read = data.data().decode('windows-1251')
        except Exception as e:
            self.log(str(e), 'err')
            self.workerbreak = True
            return

        self.readline_data += read
        indx = self.readline_data.rfind('\r\n')
        if indx != -1:
            self.ready_readline(self.readline_data[:indx])
            self.readline_data = self.readline_data[indx+2:]

    def ready_readline(self, data):
        self.readline_ready = True
        self.readline_signal.emit(data)

    def write_line(self, string):
        self.nqueue.join()
        self.readline_data = ''
        self.writeline_data = string
        self.nqueue.put('writelines')

    def worker(self):
        while True:
            item = self.nqueue.get()

            with threading.Lock():

                if item == 'writelines':
                    for s in self.writeline_data.split('\r\n'):
                        if self.workerbreak == True:
                            break
                        # write line signal
                        self.readline_ready = False
                        self.writeline_signal.emit(s + '\r\n')
                        st = time()
                        # wait nodemcu respond
                        while self.readline_ready is not True:
                            if time() - st > 4:
                                print('Respond timeout ):', 'err')
                                self.workerbreak = True
                                break
                        sleep(self.linedelay/1000)

                    # reset read state and r/w lines data
                    self.readline_ready = False
                    self.workerbreak = False
                    self.writeline_data = ''

            self.nqueue.task_done()


class ESP(QCoreApplication):
    """docstring for ESP"""
    _api_file = {
        'MPY': os.path.join(os.path.dirname(__file__), __api_mpy_path__),
        'NODE': os.path.join(os.path.dirname(__file__), __api_node_path__)
    }

    def __init__(self, serialconfig, platform='USER', api_file=None, loglevel=0):
        super(ESP, self).__init__([])
        #
        self.platform = platform
        #
        if platform in ESP._api_file.keys():
            api_file = ESP._api_file[platform]
            try:
                import json
                with open(api_file, 'r') as f:
                    self.api = json.loads(f.read())
            except Exception as e:
                raise e('error load api json file, key - '+platform)
        elif api_file is None:
            raise BaseException('wrong api key or api json file')
        #
        self.txfer = b''
        self.rxfer = b''
        #
        self.serialport = SerialPort(serialconfig)
        #
        self.serialport.readline_signal.connect(
            self.read,
            no_receiver_check=True
        )
        #
        self._response_callback = None

    def send(self, command, response_callback=None, **kwargs):
        """ """
        self._response_callback = response_callback
        # esp memory optimization, split file data
        if command == 'filewrite':
            lines = kwargs.get('data', '').split('\n')
            kwargs['data'] = ''
            for line in lines:
                ln = line.replace("'", '"') + '\\n'
                api_cmd = self.api['filewriteline']
                kwargs['data'] += api_cmd.format(data=ln) + '\r\n'
        # format command and data field
        if command != 'line':
            self.txfer = self.api[command].format(**kwargs)
        else:
            self.txfer = kwargs['data']
        # prepare to send
        self.txfer += "\r\nprint('END')"
        self.txfer.replace(r'\r', r'\\r').replace(r'\n', r'\\n')
        # send self.txfer to esp
        self.serialport.write_line(self.txfer)
        #

    def read(self, data):
        # print(self.api)
        data = data.replace('>>> ', '')
        #
        print('esp read: ', data)
        if data not in self.txfer and self._response_callback:
            self._response_callback(data)
        #
        # if self.platform == 'MPY':
        #     end = data.find('>>>')
        #     if end != -1:
        #         data = data[:end]
        #
        # if len(data) and self._response_callback:
        #     self._response_callback(data)




import os
from setuptools import setup
from setuptools import find_packages

PACKAGE = "pyesp"

setup(
    name=PACKAGE,
    version=__import__(PACKAGE).__version__,
    author="Vladislav Kamenev",
    author_email="wladkam@mail.com",
    url="https://github.com/LeftRadio/pyesp",
    description="tool for programming/comunicate with ESP8266 platforms based on MircoPython and NodeMCU",
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.md')).read(),
    package_data={
        PACKAGE: [
            os.path.join(
                os.path.dirname(__import__(PACKAGE).__api_mpy_path__),
                "*.json"
            ),
            os.path.join(os.path.dirname(
                __import__(PACKAGE).__serial_config_path__),
                "*.json"
            ),
        ]
    },
    packages=find_packages(),
    entry_points={
        "console_scripts": [
            "pyesp = pyesp.run : main"
        ]
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Utilities',
        'Programming Language :: Python :: 3',
    ],
    license='MIT'
)
